from ConfigParser import ConfigParser
from os.path import isfile

class DictMatch(object):

    def __init__(self, config=None, mapped_options={}, mapped_defaults={}):
        self.config = config 
        self.mapped_options = mapped_options
        self.mapped_defaults = mapped_defaults

    def options(self):
        # If all fails we will always have default values
        configuration = self.defaults()

        try:
            if self.config == None or isfile(self.config) == False:
                configuration = self.defaults()
                return configuration

        except TypeError:
            # if we are getting a ready-to-go dict then we still try 
            # to do our little translation-and-map thing and if that 
            # comes out as empty, then we assume keys are already 
            # translated
            if type(self.config) is dict:
                configuration = self.key_matcher(self.config, return_empty=True)
                if not configuration:
                    configuration = self.defaults(self.config)
                return configuration

            # we could get an object that is dict-like but type(object)
            # doesn't recognize it as a dict 
            else:
                configuration = self.key_matcher(self.config)
                return configuration
        
        else: # this will get executed *only* if we are seeing a file
            try:
                parser = ConfigParser()
                parser.read(self.config)
                file_options = parser.defaults()
                configuration = self.key_matcher(file_options)
            except Exception, error:
                raise OptionConfigurationError(error)

        return configuration


    def key_matcher(self, original, return_empty=False):
        converted_opts = {}

        for key, value in self.mapped_options.items():
            try:
                file_value = original[key]
                converted_opts[value] = file_value
            except KeyError:
                pass # we will fill any empty values later with config_defaults

        if len(converted_opts) == 0 and return_empty == True:
            return False
        try:
            configuration = self.defaults(converted_opts)
            return configuration
        except Exception, error:
            raise OptionConfigurationError(error)


    def defaults(self, config=None):
        """From the config dictionary it checks missing values and
        adds the defaul ones for them if any"""
        if config == None:
            return self.mapped_defaults

        for key in self.mapped_defaults.keys():
            try:
                config[key]
                if config[key] == '':
                    config[key] = self.mapped_defaults[key]
            except KeyError:
                config[key] = self.mapped_defaults[key]
        return config


class OptionConfigurationError(Exception):
    """Base class for exceptions in this module."""
    pass



import os
import unittest

from guachi import ConfigMapper
from guachi.database import dbdict

# An example of a default config dict
DEFAULT_CONFIG = {
            'frequency': 60,
            'master': 'False',
            'host': 'localhost',
            'ssh_user': 'root',
            'ssh_port': 22,
            'hosts_path': '/opt/pacha',
            'hg_autocorrect': 'True',
            'log_enable': 'False',
            'log_path': 'False',
            'log_level': 'DEBUG',
            'log_format': '%(asctime)s %(levelname)s %(name)s %(message)s',
            'log_datefmt' : '%H:%M:%S'
            }


class test_ConfigMapper(unittest.TestCase):

    def setUp(self):
        try:
            os.remove('/tmp/guachi.db')
            os.remove('/tmp/foo_guachi.db')
        except Exception:
            pass 

    def tearDown(self):
        try:
            os.remove('/tmp/guachi.db')
            os.remove('/tmp/foo_guachi.db')
        except Exception:
            pass 


    def test_init(self):
        foo = ConfigMapper('/tmp')
        expected = '/tmp/guachi.db'
        actual = foo.path 
        self.assertEqual(actual, expected) 


    def test__call__(self):
        """ConfigMapper should be callable"""
        foo = ConfigMapper('/tmp')
        actual = foo()
        expected = {}
        self.assertEqual(actual, expected) 


    def test_set_ini_options(self):
        foo = ConfigMapper('/tmp')
        my_config = {'config.db.port':'db_port'}
        foo.set_ini_options(my_config)
        db = dbdict(path='/tmp/guachi.db', table='_guachi_options')
        expected = my_config
        actual = db.get_all()
        self.assertEqual(actual, expected) 


    def test_set_default_options(self):
        foo = ConfigMapper('/tmp')
        my_config = {'db_port':1234}
        foo.set_default_options(my_config)
        db = dbdict(path='/tmp/guachi.db', table='_guachi_defaults')
        expected = my_config
        actual = db.get_all()
        self.assertEqual(actual, expected) 


    def test_get_ini_options(self):
        foo = ConfigMapper('/tmp')
        my_config = {'config.db.port':'db_port'}
        foo.set_ini_options(my_config)
        defaults = foo.get_ini_options()
        actual = defaults['config.db.port']
        expected = 'db_port'
        self.assertEqual(actual, expected) 


    def test_get_default_options(self):
        foo = ConfigMapper('/tmp')
        my_config = {'db_port':1234}
        foo.set_default_options(my_config)
        defaults = foo.get_default_options()
        actual = defaults['db_port']
        expected = 1234
        self.assertEqual(actual, expected) 


    def test_path_verify_file(self):
        foo = ConfigMapper('/tmp/foo_guachi.db')
        actual = foo._path_verify('/tmp/foo_guachi.db')
        expected = '/tmp/foo_guachi.db'
        self.assertEqual(actual, expected) 


    def test_path_verify_dir(self):
        foo = ConfigMapper('/tmp')
        actual = foo._path_verify('/tmp')
        expected = '/tmp/guachi.db'
        self.assertEqual(actual, expected) 

    def test_update_config_dict_empty(self):
        """Pass an empty dict and get defaults back"""
        foo = ConfigMapper('/tmp')
        foo.update_config({})
        db = dbdict('/tmp/guachi.db')
        actual = db.get_all()
        expected ={}
        self.assertEqual(actual, expected) 
        

    def test_update_config_dict(self):
        foo = ConfigMapper('/tmp')
        foo.update_config(DEFAULT_CONFIG)
        db = dbdict('/tmp/guachi.db')
        actual = db.get_all()
        expected = DEFAULT_CONFIG
        self.assertEqual(actual, expected) 
 
        
    def test_set_config_dict_empty(self):
        """Pass an empty dict and get defaults back"""
        foo = ConfigMapper('/tmp')
        foo.set_config({})
        db = dbdict('/tmp/guachi.db')
        actual = db.get_all()
        expected ={}
        self.assertEqual(actual, expected) 
        

    def test_set_config_dict(self):
        foo = ConfigMapper('/tmp')
        foo.set_config(DEFAULT_CONFIG)
        db = dbdict('/tmp/guachi.db')
        actual = db.get_all()
        expected = DEFAULT_CONFIG
        self.assertEqual(actual, expected) 
 

    def test_get_dict_config(self):
        foo = ConfigMapper('/tmp')
        foo.set_config(DEFAULT_CONFIG)
        actual = foo.get_dict_config()
        expected = DEFAULT_CONFIG
        self.assertEqual(actual, expected) 


    def test_integrity_check(self):
        foo = ConfigMapper('/tmp')
        foo.set_config(DEFAULT_CONFIG)
        actual = foo.integrity_check()
        self.assertTrue(actual) 


    def test_stored(self):
        foo = ConfigMapper('/tmp')
        bar = foo.stored_config()
        self.assertEqual(bar, {})


    def test_stored_get_data(self):
        """Verify we can actually get data by using stored"""
        foo = ConfigMapper('/tmp')
        bar = foo.stored_config()
        bar['a'] = 1
        self.assertEqual(bar, {'a':1})





import os
import sqlite3
import unittest
from guachi import database

class TestDbdict(unittest.TestCase):
    
    def tearDown(self):
        try:
            os.remove('/tmp/test_guachi')
        except Exception:
            pass 

    def test_create_database(self):
        foo = database.dbdict('/tmp/test_guachi')
        self.assertTrue(os.path.isfile('/tmp/test_guachi'))

    def test_init(self):
        foo = database.dbdict('/tmp/test_guachi')

        self.assertEqual(foo.db_filename, '/tmp/test_guachi')
        self.assertEqual(foo.table, '_guachi_data')
        self.assertEqual(foo.select_value, 'SELECT value FROM _guachi_data WHERE key=?')
        self.assertEqual(foo.select_key, 'SELECT key FROM _guachi_data WHERE key=?')
        self.assertEqual(foo.update_value, 'UPDATE _guachi_data SET value=? WHERE key=?')
        self.assertEqual(foo.insert_key_value, 'INSERT INTO _guachi_data (key,value) VALUES (?,?)')
        self.assertEqual(foo.delete_key, 'DELETE FROM _guachi_data WHERE key=?')

    def test_init_guachi_table(self):
        """Make sure we can check other tables"""
        foo = database.dbdict('/tmp/test_guachi', table='_guachi_options')
        self.assertEqual(foo.table, '_guachi_options')

    def test_get_item_keyerror(self):
        foo = database.dbdict('/tmp/test_guachi')
        self.assertRaises(KeyError, foo.__getitem__, 'meh')

    def test_get_item(self):
        foo = database.dbdict('/tmp/test_guachi')
        foo['bar'] = 'beer'
        self.assertEqual(foo['bar'], u'beer')
 
    def test_setitem_update(self):
        """If it already exists, you need to do an update"""
        foo = database.dbdict('/tmp/test_guachi')
        foo['a'] = 1
        foo['a'] = 2
        self.assertEqual(foo['a'], 2)

    def test_close_db(self):
        foo = database.dbdict('/tmp/test_guachi')
        foo['bar'] = 'beer'
        foo._close()
        self.assertRaises(sqlite3.ProgrammingError, foo.__setitem__, 'bar', {'a':'b'})

    def test_setitem_typeerror(self):
        foo = database.dbdict('/tmp/test_guachi')
        self.assertRaises(sqlite3.InterfaceError, foo.__setitem__, 'bar', {'a':'b'})

    def test_delitem_keyerror(self):
        foo = database.dbdict('/tmp/test_guachi')
        self.assertRaises(KeyError, foo.__delitem__, 'meh')

    def test_delitem(self):
        foo = database.dbdict('/tmp/test_guachi')
        foo['bar'] = 'beer'
        self.assertEqual(foo['bar'], 'beer')
        del foo['bar']
        self.assertRaises(KeyError, foo.__delitem__, 'bar')

    def test_key_empty(self):
        foo = database.dbdict('/tmp/test_guachi')
        self.assertEqual(foo.keys(), [])

    def test_keys(self):
        foo = database.dbdict('/tmp/test_guachi')
        foo['bar'] = 'beer'
        self.assertEqual(foo.keys(), ['bar'])

    def test_integrity_check_true(self):
        foo = database.dbdict('/tmp/test_guachi')
        self.assertTrue(foo._integrity_check())

    def test_integrity_check_false(self):
        foobar = open('/tmp/test_guachi', 'w')
        foobar.write('meh')
        foobar.close()
        foo = database.dbdict('/tmp/test_guachi')
        self.assertEquals(foo._integrity_check()[0], 'file is encrypted or is not a database')

        



from guachi.database import dbdict
from guachi.config import DictMatch
import os

class ConfigMapper(object):

    def __init__(self, path):
        self.path = self._path_verify(path)


    def __call__(self):
        db = dbdict(self.path)
        return db


    def set_ini_options(self, dictionary):
        """Maps your INI configuration keys to dictionary keys"""
        db = dbdict(self.path, table='_guachi_options') 
        for key, value in dictionary.items():
            db[key] = value 


    def set_default_options(self, dictionary):
        """Maps the default values that we can fill in if keys are empty"""
        db = dbdict(self.path, table='_guachi_defaults') 
        for key, value in dictionary.items():
            db[key] = value


    def get_ini_options(self):
        """Returns the dictionary that maps INI style options to 
        dictionary options"""
        db = dbdict(self.path, table='_guachi_options') 
        return db


    def get_default_options(self):
        """Returns the default options we hold"""
        db = dbdict(self.path, table='_guachi_defaults') 
        return db


    def set_config(self, configuration=None):
        """Accepts a dictionary or a file to set persistent configurations"""
        mapped_ini = self.get_ini_options()
        mapped_defaults = self.get_default_options()

        # First make sure that whatever we get, gets translated
        # into a dictionary 
        dict_match = DictMatch(configuration, mapped_ini, mapped_defaults)
        dict_config = dict_match.options()
        if len(dict_config.items()) > 0:
            db = dbdict(self.path)
            for key, value in dict_config.items():
                db[key] = value 


    def update_config(self, configuration=None):
        """An alias to set_config for readability sake"""
        return self.set_config(configuration)


    def get_dict_config(self):
        """Returns the full stored configuration values as a dictionary"""
        db = dbdict(self.path)
        return db.get_all()


    def stored_config(self):
        """Access the DB directly for pay-as-you-go configuration needs
        You get an empty dict but with access to any keys when requested
        """
        db = dbdict(self.path)
        return db


    def integrity_check(self):
        """Verify the database is OK"""
        db = dbdict(self.path)
        return db._integrity_check()


    def _path_verify(self, path):
        """The need to have valid absolute paths"""
        if not os.path.isdir(path):
            return os.path.abspath(path)
        if os.path.isdir(path):
            abspath = os.path.abspath(path)
            return abspath+'/guachi.db'




from os import path, remove, mkdir
import unittest 

from guachi import ConfigMapper


class TestIntegration(unittest.TestCase):

    def setUp(self):
        self.mapped_options = {
            'guachi.db.host':'db_host',
            'guachi.db.port':'db_port',
            'guachi.web.host':'web_host',
            'guachi.web.port':'web_port',
            }

        self.mapped_defaults = {
            'db_host': 'localhost',
            'db_port': 27017,
            'web_host': 'localhost',
            'web_port': '8080',
            }

        try:
            if path.exists('/tmp/guachi'):
                remove('/tmp/guachi')
            else:
                mkdir('/tmp/guachi')
        except Exception:
            pass

    def tearDown(self):
        try:
            if path.exists('/tmp/guachi'):
                remove('/tmp/guachi')
            else:
                mkdir('/tmp/guachi')
        except Exception:
            pass



    def test_access_mapped_configs_empty_dict(self):
        foo = ConfigMapper('/tmp/guachi')
        foo.set_ini_options(self.mapped_options)
        foo.set_default_options(self.mapped_defaults)
        foo.set_config({})

        # try as much operations as we can and assert them
        self.assertEqual(foo(), {})
        self.assertEqual(foo.path, '/tmp/guachi/guachi.db')
        self.assertEqual(foo.get_ini_options(), {})
        self.assertEqual(foo.get_default_options(), {})
        self.assertEqual(foo.get_dict_config(), self.mapped_defaults)
        self.assertEqual(foo.stored_config(), {})
        self.assertTrue(foo.integrity_check())


    def test_access_mapped_configs_dict(self):
        foo = ConfigMapper('/tmp/guachi')
        foo.set_ini_options(self.mapped_options)
        foo.set_default_options(self.mapped_defaults)
        foo.set_config({'db_host':'example.com', 'db_port':0})

        # try as much operations as we can and assert them
        self.assertEqual(foo(), {})
        self.assertEqual(foo.path, '/tmp/guachi/guachi.db')
        self.assertEqual(foo.get_ini_options(), {})
        self.assertEqual(foo.get_default_options(), {})
        self.assertEqual(foo.get_dict_config(), 
                {u'web_port': u'8080', 
                 u'web_host': u'localhost', 
                 u'db_host': u'example.com', 
                 u'db_port': 0})
        self.assertEqual(foo.stored_config(), {})
        self.assertTrue(foo.integrity_check())



import sqlite3

BASE = """CREATE TABLE IF NOT EXISTS _guachi_data (key PRIMARY KEY, value)"""
OPT_MAP = """CREATE TABLE IF NOT EXISTS _guachi_options (key PRIMARY KEY, value)"""
DEF_MAP = """CREATE TABLE IF NOT EXISTS _guachi_defaults (key PRIMARY KEY, value)""" 

class dbdict(dict):
    
    def __init__(self, path, table='_guachi_data'):
        dict.__init__(self)
        self.table = table
        self.db_filename = path
        self.con = sqlite3.connect(self.db_filename)
        self.con.execute(BASE)
        self.con.execute(OPT_MAP)
        self.con.execute(DEF_MAP)

        self.select_value = "SELECT value FROM %s WHERE key=?" % self.table 
        self.select_key = "SELECT key FROM %s WHERE key=?" % self.table 
        self.update_value = "UPDATE %s SET value=? WHERE key=?" % self.table
        self.insert_key_value = "INSERT INTO %s (key,value) VALUES (?,?)" % self.table 
        self.delete_key = "DELETE FROM %s WHERE key=?" % self.table
        self.select_keys = "SELECT key from %s" % self.table
        self.select_all = "SELECT * from %s" % self.table


    def __getitem__(self, key):
        row = self.con.execute(self.select_value,(key,)).fetchone()
        if not row: raise KeyError, "key '%s' not found in persistent dictionary" % key
        return row[0]
    

    def __setitem__(self, key, item):
        try:
            if self.con.execute(self.select_key, (key,)).fetchone():
                self.con.execute(self.update_value, (item,key))
            else:
                self.con.execute(self.insert_key_value,(key, item))
        except sqlite3.InterfaceError, e:
            raise sqlite3.InterfaceError(e)

        self.con.commit()
        return dict.__setitem__(self, key, item)
               

    def __delitem__(self, key):
        if self.con.execute(self.select_key ,(key,)).fetchone():
            self.con.execute(self.delete_key ,(key,))
            self.con.commit()
        else:
             raise KeyError, "key '%s' not found in persistent dictionary" % key 
             

    def keys(self):
        return [row[0] for row in self.con.execute(self.select_keys).fetchall()]


    def items(self):
        return [row for row in self.con.execute(self.select_all).fetchall()]


    def get_all(self):
        """If you need to get absolutely everything at once - ugh.
        This can get really expensive and defeats the purpose but oh well"""
        dict_all = {}
        for key, value in self.con.execute(self.select_all):
            dict_all[key] = value 
        return dict_all


    def _integrity_check(self):
        """Make sure we are doing OK"""
        try:
            integrity = self.con.execute("pragma integrity_check").fetchone()
            if integrity == (u'ok',):
                return True
        except Exception, error:
            return error


    def _close(self):
        self.con.close()



import unittest
from os import remove, mkdir, path

from guachi.config  import DictMatch, OptionConfigurationError 

class MockDict(dict):
    pass

def setup():
    try:
        if path.exists('/tmp/guachi'):
            remove('/tmp/guachi')
        else:
            mkdir('/tmp/guachi')
    except Exception:
        pass

    txt = open('/tmp/guachi/conf.ini', 'w')
    text = """
[DEFAULT]
# Middleware Configuration
guachi.middleware.server_id = 2
guachi.middleware.application = secondary

# Database (Mongo)
guachi.db.host = remote.example.com
guachi.db.port = 00000

# Web Interface
guachi.web.host = web.example.com
guachi.web.port = 80

# Logging
guachi.log.level = DEBUG
guachi.log.datefmt = %H:%M:%S
guachi.log.format = %(asctime)s %(levelname)s %(name)s  %(message)s

# Cache
guachi.cache = 10
    """
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_two.ini', 'w')
    text = """
[DEFAULT]
# Middleware Configuration
guachi.middleware.application = secondary

# Database (Mongo)
guachi.db.host = remote.example.com
guachi.db.port = 00000

# Web Interface
guachi.web.host = web.example.com
guachi.web.port = 80

# Logging
guachi.log.level = DEBUG
guachi.log.datefmt = %H:%M:%S
guachi.log.format = %(asctime)s %(levelname)s %(name)s  %(message)s    

# Cache
guachi.cache = 10
"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_three.ini', 'w')
    text = """
[DEFAULT]
# Middleware Configuration
guachi.middleware.server_id = 2
guachi.middleware.application = secondary

# Database (Mongo)
guachi.db.port = 00000

# Web Interface
guachi.web.host = web.example.com
guachi.web.port = 80

# Logging
guachi.log.level = DEBUG
guachi.log.datefmt = %H:%M:%S
guachi.log.format = %(asctime)s %(levelname)s %(name)s  %(message)s    

# Cache
guachi.cache = 10
"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_four.ini', 'w')
    text = """
[DEFAULT]
# Middleware Configuration
guachi.middleware.server_id = 2
guachi.middleware.application = secondary

# Database (Mongo)
guachi.db.host = remote.example.com

# Web Interface
guachi.web.host = web.example.com
guachi.web.port = 80

# Logging
guachi.log.level = DEBUG
guachi.log.datefmt = %H:%M:%S
guachi.log.format = %(asctime)s %(levelname)s %(name)s  %(message)s    

# Cache
guachi.cache = 10
"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_five.ini', 'w')
    text = """
[DEFAULT]
# Middleware Configuration
guachi.middleware.server_id = 2

# Database (Mongo)
guachi.db.host = remote.example.com
guachi.db.port = 00000

# Web Interface
guachi.web.host = web.example.com
guachi.web.port = 80

# Logging
guachi.log.level = DEBUG
guachi.log.datefmt = %H:%M:%S
guachi.log.format = %(asctime)s %(levelname)s %(name)s  %(message)s    

# Cache
guachi.cache = 10
"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_six.ini', 'w')
    text = """
[DEFAULT]
# Middleware Configuration
guachi.middleware.server_id = 2
guachi.middleware.application = secondary

# Database (Mongo)
guachi.db.host = remote.example.com
guachi.db.port = 00000

# Web Interface
guachi.web.port = 80

# Logging
guachi.log.level = DEBUG
guachi.log.datefmt = %H:%M:%S
guachi.log.format = %(asctime)s %(levelname)s %(name)s  %(message)s    

# Cache
guachi.cache = 10
"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_seven.ini', 'w')
    text = """
[DEFAULT]

# Database (Mongo)
guachi.db.host = remote.example.com
guachi.db.port = 0

"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_eight.ini', 'w')
    text = """
[DEFAULT]
# Database (Mongo)
guachi.db.host = example.com 
guachi.db.port = 

# Web Interface
guachi.web.host = 
guachi.web.port = 


"""
    txt.write(text)
    txt.close()

    txt = open('/tmp/guachi/conf_nine.ini', 'w')
    text = """
[DEFAULT]
# Database (Mongo)
guachi.db.host = 
guachi.db.port = 

# Web Interface
guachi.web.host = 
guachi.web.port = 

"""
    txt.write(text)
    txt.close()


def teardown():
    try:
        remove('/tmp/guachi') 
    except Exception:
        pass


class TestConfigOptions(unittest.TestCase):

    def setUp(self):
        self.mapped_options = {
            'guachi.db.host':'db_host',
            'guachi.db.port':'db_port',
            'guachi.web.host':'web_host',
            'guachi.web.port':'web_port',
            }

        self.mapped_defaults = {
            'db_host': 'localhost',
            'db_port': 27017,
            'web_host': 'localhost',
            'web_port': '8080',
            }


    def test_options_config_none_empty_defaults(self):
        """No config and no defaults should return an empty dict"""
        opts = DictMatch()
        actual = opts.options()
        expected = {}
        self.assertEqual(actual, expected)


    def test_options_config_invalid_empty_defaults(self):
        """Invalid config file and no defaults should return an empty dict"""
        opts = DictMatch(config='/path/to/invalid/file')
        actual = opts.options()
        expected = {}
        self.assertEqual(actual, expected)


    def test_options_config_dict_empty_defaults(self):
        """A dict config and no defaults should return an empty dict"""
        opts = DictMatch(config={})  
        actual = opts.options()
        expected = {}
        self.assertEqual(actual, expected)


    def test_options_from_dict(self):
        """Pass a dict with no values and get defaults back"""
        opt = DictMatch(config={}, mapped_defaults=self.mapped_defaults)
        actual = opt.options()
        expected = self.mapped_defaults
        self.assertEqual(actual, expected) 


    def test_options_dict_like_object(self):
        """Pass a dict-like object and make sure we get valid options back"""
        mock_dict = MockDict() 
        opt = DictMatch(config=mock_dict, mapped_defaults=self.mapped_defaults)
        actual = opt.options()
        expected = self.mapped_defaults
        self.assertEqual(actual, expected) 


    def test_options_from_file_empty_options(self):
        """A conf file with empty values should get values filled in"""
        opt = DictMatch('/tmp/guachi/conf_nine.ini', self.mapped_options, self.mapped_defaults)
        actual = opt.options()
        expected = self.mapped_defaults
        self.assertEqual(actual, expected) 


    def test_options_from_file_one_option(self):
        """A conf file with one value should get values filled in"""
        opt = DictMatch('/tmp/guachi/conf_eight.ini', self.mapped_options, self.mapped_defaults)
        actual = opt.options()
        expected = {
            'db_host': 'example.com',
            'db_port': 27017,
            'web_host': 'localhost',
            'web_port': '8080',
            }

        self.assertEqual(actual, expected) 

    
    def test_options_from_file_empty_defaults(self):
        """Just one default should not overwrite other config values"""
        opt = DictMatch('/tmp/guachi/conf_eight.ini', self.mapped_options, {})
        actual = opt.options()
        expected = {
            'db_host': 'example.com',
            'db_port': '',
            'web_host': '',
            'web_port': '',
            }

        self.assertEqual(actual, expected) 


    def test_options_key_error_passes(self):
        """When options are missing options() passes on the KeyError"""
        opt = DictMatch('/tmp/guachi/conf_seven.ini', self.mapped_options, self.mapped_defaults)
        actual = opt.options()
        expected = {
            'db_host': 'remote.example.com',
            'db_port': '0',
            'web_host': 'localhost',
            'web_port': '8080',
            }

        self.assertEqual(actual, expected) 

        
    def test_options_from_file_raise_error(self):
        """Error out if we are passing a string in defaults"""
        opt = DictMatch('/tmp/guachi/conf_eight.ini', self.mapped_options, '')   
        self.assertRaises(OptionConfigurationError, opt.options) 


    def test_options_raise_error_mapped_options(self):
        """Error out if we are passing a None object in defaults"""
        opt = DictMatch('/tmp/guachi/conf_eight.ini', None, self.mapped_defaults)   
        self.assertRaises(OptionConfigurationError, opt.options) 


if __name__ == '__main__':
    unittest.main()



import distribute_setup
distribute_setup.use_setuptools()
from setuptools import setup

tests_require = ['nose']

setup(
    name = "guachi",
    version = "0.0.5",
    packages = ['guachi'],
    include_package_data=True,
    package_data = {
        '': ['distribute_setup.py'],
        },

    # metadata 
    author = "Alfredo Deza",
    author_email = "alfredodeza [at] gmail [dot] com",
    description = "Global, persistent configurations as dictionaries",
    long_description = """\
When projects start to grow, the need for a globally accessible configuration
manager is obvious.

Having configurations mapped to dictionaries is really useful, but can create a 
problem with memory.

**Guachi** not only holds persistent dictionaries on disk, but it also maps 
INI style keys to dictioanry keys, and can fill in the default values if some 
of them are missing.

You do not need to know anything about how **guachi** stores the values, just 
treat it like a regular dictionary!
""",
   classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
      ],

    license = "MIT",
    keywords = "configuration management persistent dictionaries dictionary parse map mapping",
    url = "http://code.google.com/p/guachi",   

)

